function [intercept,theta,obj,gap] = logist_lars (x, y, nrfeat, pobj, lambda, tol, large)

if(~exist('large'))
   large = 0;
end

[nsamples, nfeatures] = size(x);


theta = sparse ( zeros (nfeatures,1) );

intercept = 0;

max_iter = 200;
status = 0;

A = sparse(1:nsamples,1:nsamples,y)*x;

beta = 0.9;alpha = 0.3;


z2 = x*theta + intercept;z3 = 1+exp(-z2.*y);
pre_nll = sum( log(z3) )/nsamples;

for iter = 1 : max_iter

    z0 = 1./(1+exp(-z2));z1 = z0.*(1-z0);
    z5 = ( 1 - 1./z3 ) .* y;z4 = z2 + ( z5 ./ z1);

    A1 = sparse(1:nsamples,1:nsamples,sqrt(z1));

    z1s = sqrt(z1);z1snorm2 = z1s'*z1s;

    atmp = A1*x;btmp = A1*(z4-intercept);

    if(large)
       preobj = sum( log(z3) );
       istp = sum( z5 ) / sum(z1);
       if(iter==1)
          theta_new = lars(atmp, btmp, 'lasso', nrfeat, 400)';
       else
	  theta_new = lars(atmp, btmp, 'lasso', nrfeat)';
       end
       stp = theta_new-theta;
       t = 1;eps = sum((z5'*x)*stp);
       for biter = 1 : 500
          theta_new = theta + t*stp;
	  postobj = sum( log(1+exp(-(x*theta_new+intercept).*y)) );
	  if( postobj < preobj - alpha * t * eps )
	     break;
	  else  
             t = beta * t;%fprintf('%g %g %g\n',preobj,postobj, preobj - alpha*t*eps);
	  end
       end
       theta = theta + t*stp;
       t = 1;eps = sum(z5)*istp; 
       for biter = 1 : 500
          intercept_new = intercept + t*istp;
	  postobj = sum( log(1+exp(-(x*theta+intercept_new).*y)) );
	  if( postobj < preobj - alpha * t * eps )
	     break;
	  else
	     t = beta * t;
	  end
       end
       intercept = intercept + t * istp;
    else
       a = -z1s*(z1s'*atmp)./z1snorm2 + atmp;
       b = -z1s*(z1s'*btmp)./z1snorm2 + btmp; 
       theta = lars(a, b, 'lasso', nrfeat)';
       intercept = -z1s'*(A1*x*theta-A1*z4)/z1snorm2;
    end

    z2 = x*theta + intercept;z3 = 1+exp(-z2.*y);

    post_nll = sum( log(z3) )/nsamples;

    prog = pre_nll-post_nll;

    gap = post_nll-pobj;
    
    pre_nll = post_nll;
    fprintf('pobj = %g,	ll = %g,   gap = %g,	progress = %g\n',pobj, post_nll, gap, prog);
    
    if (gap < tol)	%% stop criteria: now is the gap with the primal objective
        status  = 1;
        break;
    end
    
end
if (status == 1)
    disp('ok.')
else
    disp('fail.')
end
obj = pobj;



